/*
This file is part of the Project JsBuild

JsBuild - http://projects.stueckseln.de/jsbuild/
JsBuild is a project released under the MIT license.


Copyright (c) 2009 Thomas Lehmann

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package de.stueckseln.buildtools.jsbuild;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.tools.ant.DirectoryScanner;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import de.stueckseln.buildtools.utils.Logger;
import de.stueckseln.buildtools.utils.PathBuilder;
import de.stueckseln.buildtools.utils.SimplePathBuilder;

/** Concrete class implementing parseModules()
  * parseModules() will now load an xml-based configuration
  * 
  * @author Thomas Lehmann
  *
  */
public class JsBuilder extends AbstractJsBuilder {
	private PathBuilder srcPathBuilder;	
	
	public void setSourceDirectory(String sourceDirectory) {
		this.sourceDirectory = sourceDirectory;
	}
	
	/** Initialize this instance without setting descriptor and directories and default output System.out
	  * 
	  */
	public JsBuilder() throws JsBuilderException {	
		super();
		
		this.srcPathBuilder = new SimplePathBuilder();
	}

	/** Initialize this instance without setting descriptor and directories and custom Logger
	  * @param logger to useo
	  */
	public JsBuilder(Logger logger) throws JsBuilderException {	
		super(logger);
		
		this.srcPathBuilder = new SimplePathBuilder();
	}

	/** Initialize this instance with setting descriptor and directories and default output System.out
	  * 
	  */
	public JsBuilder(String sourceDir, String targetDir, String modulesFile) throws JsBuilderException {
		this();
	
		this.setSourceDirectory(sourceDir);
		this.setTargetDirectory(targetDir);
		
		this.setModulesFile(modulesFile);		
	}

	/** Initialize this instance with setting descriptor and directories and custom Logger
	  * @param logger to use
	  */
	public JsBuilder(String sourceDir, String targetDir, String modulesFile, Logger logger) throws JsBuilderException {
		this(logger);
	
		this.setSourceDirectory(sourceDir);
		this.setTargetDirectory(targetDir);
		
		this.setModulesFile(modulesFile);		
	}
	
	/** Converts an package name into a path 
	  * @param pkg package name in dot notation
	  * @return a package path in slash notation
	  */
	private static String packageToPath(String pkg){
		return pkg.replaceAll("\\.", "/");
	}
	
	/* Process a <module> node, the supplied parameter has to be the module node 
	 * 
	 */
	private JsModule processModule(Node moduleNode) throws JsBuilderException {
		int moduleNum = this.modules.size()+1; //current module (based on count of recently processed)
		String moduleName = null;
		String modulePackage = null;
		Node filesSibling = null;
		NamedNodeMap moduleAttrs = null;
		String filesNodename = null;
		
		moduleAttrs = moduleNode.getAttributes();
		
		
		
		if (moduleAttrs.getNamedItem("name") == null){
			throw new JsBuilderException("Module #"+ moduleNum +" lacks mandatory attribute name." );
		}

		moduleName = moduleAttrs.getNamedItem("name").getNodeValue().trim();

		if (moduleName.length() < 1) {
			throw new JsBuilderException("Name of module #"+ moduleNum +" is too short (minimum length is 1)." );
		}
		
		if (moduleAttrs.getNamedItem("package") == null){
			throw new JsBuilderException("Module '" + moduleName + "' lacks mandatory attribute package." );
		}
	
		modulePackage = moduleAttrs.getNamedItem("package").getNodeValue().trim();

		if (modulePackage.length() < 1) {
			throw new JsBuilderException("Package name of module #"+ moduleNum +" is too short (minimum length is 1)." );
		}
		
		filesSibling = moduleNode.getFirstChild();
		
		JsModule module = new JsModule(moduleName);
		
		while (filesSibling != null){
			filesNodename = filesSibling.getNodeName();
		
			if (filesNodename.equals("file") || filesNodename.equals("fileset")) {				
				if (filesNodename.equals("file")) {						
					//LOG.debug("file found");
					
					processFile(filesSibling, module, modulePackage);
				} else if (filesNodename.equals("fileset")) {
					//LOG.debug("fileset found");
					
					processFileset(filesSibling.getFirstChild(), module, modulePackage);
				}
			}
			
			filesSibling = filesSibling.getNextSibling();
		}
		
		return module;
	}
	
	private void processFileset(Node cludeSibling, JsModule module, String pkg) throws JsBuilderException {
		Node filesetNode = cludeSibling.getParentNode();
		NamedNodeMap filesetAttrs = filesetNode.getAttributes();
		String cludeNodeName = null; 

		String path = null;
		
		String files = null, excludes = null, includes = null;
		boolean hasEitherClude = false;
		Node filesAttr = null;
		
		if (filesetNode.hasAttributes() && filesetAttrs.getNamedItem("path") != null) {
			path = filesetAttrs.getNamedItem("path").getNodeValue();
		}
				
		// walk through all possible include/exclude nodes
		while (cludeSibling != null){
			cludeNodeName = cludeSibling.getNodeName();
			
			if (cludeNodeName == "exclude" || cludeNodeName == "include"){
				if (cludeSibling.getAttributes().getNamedItem("files") != null){
					filesAttr = cludeSibling.getAttributes().getNamedItem("files");	
					files = filesAttr.getNodeValue(); 
				} else {
					files = null;
				}
				
				if (files != null && files.length() == 0) {
					files = null;
				}

				if (cludeNodeName == "exclude"){
					excludes = files;
				} else if (cludeNodeName == "include") {
					includes = files;
				}			
			}
			
			cludeSibling = cludeSibling.getNextSibling();
		}
				
		if (path == null){
			path = ".";
		}
		
		createFileset(module, path, pkg, includes, excludes);
	}
	
	
	private void processFile(Node fileNode, JsModule module, String pkg) throws JsBuilderException {
		NamedNodeMap fileAttrs = fileNode.getAttributes();
		String path = null;
		String pkgPath = JsBuilder.packageToPath(pkg);
		
		if (fileAttrs.getNamedItem("path") == null){
			throw new JsBuilderException("file-element lacks mandatory path attribute.");
		}
		
		path = fileAttrs.getNamedItem("path").getNodeValue();
	
		if (path == null || path.length() == 0){
			throw new JsBuilderException("Length of path is too short for module " + module.getName() + ".");
		}
		
		this.srcPathBuilder.init(this.getSourceDirectory());
		this.srcPathBuilder.append(pkgPath);
		this.srcPathBuilder.append(path);
		
		File file = new File(this.srcPathBuilder.getPath());
		
		if (!file.exists() || !file.canRead()){
			throw new JsBuilderFileException("Specified file " + file + " for module " + module.getName() + " not existing or not readable.");
		}
		
		module.addFile(this.srcPathBuilder.getPath());
	}

	
	private void createFileset(JsModule module, String path, String pkg, String includes, String excludes){
		String[] includesArr;
		String[] excludesArr; 
		
		String pkgPath = JsBuilder.packageToPath(pkg);
		
		this.srcPathBuilder.init(this.sourceDirectory);
		this.srcPathBuilder.append(pkgPath);
		this.srcPathBuilder.append(path);
				
		DirectoryScanner ds = new DirectoryScanner();		
		ds.setBasedir(this.srcPathBuilder.getPath());
		
		if (excludes != null){
			excludesArr = excludes.split(","); 
			ds.setExcludes(excludesArr);
		}
		
		if (includes != null){
			includesArr = includes.split(","); 
			ds.setIncludes(includesArr);
		} else {
			ds.setIncludes(new String[]{"**/*"});
		}
		
		ds.scan();
		
		String[] files = ds.getIncludedFiles();
		
		String filename;
		for (int i=0; i<files.length; i++){
			filename = this.srcPathBuilder.getPath() + this.srcPathBuilder.getSeparator() + files[i];
			if (new File(filename).canRead()){
				module.addFile(filename);				
			}
		}
	
		//LOG.debug("createFileset(): ds-base=" + this.srcPathBuilder + ", includes="+includes + ", excludes="+ excludes);
		//LOG.debug("                 found " + files.length + " files");
		// throw new JsBuilderFileException		
	}
	
	protected void parseModules() throws JsBuilderException {
		DocumentBuilder builder = null;
		Element document = null;
		DocumentBuilderFactory factory = null;
		final Boolean validationError = null;
		
		List<JsModule> modules = new ArrayList<JsModule>();
		
		if (this.descriptorFile == null){
			throw new JsBuilderException("Module file not specified");
		}
		if (!(new File(this.descriptorFile).exists())) {
			throw new JsBuilderException("Module file '"+this.descriptorFile+"' does not exist.");
		}

		try {
			factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(true);
			
			builder = factory.newDocumentBuilder();
			builder.setErrorHandler(new org.xml.sax.ErrorHandler(){
				
			final Logger log = LOG;
			
				public void warning(SAXParseException e)throws SAXParseException{
					log.warning("DTD validation-waring: " + e.getMessage() + " " + e.getLineNumber() + ":" + e.getColumnNumber());
				}
				
				public void error(SAXParseException e) throws SAXParseException {
					log.error("DTD validation-error: " + e.getMessage() + " " + e.getLineNumber() + ":" + e.getColumnNumber());
					throw new SAXParseException("DTD validation failed.", e.getPublicId(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
				}
				
				public void fatalError(SAXParseException e)throws SAXException {
					log.error("DTD validation-error: " + e.getMessage() + " " + e.getLineNumber() + ":" + e.getColumnNumber());
					throw new SAXParseException("DTD validation failed.", e.getPublicId(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
				}
				
			});
			
		} catch (Exception e) {
			throw new JsBuilderException("Cannot get document builder instance.");
		}
		
		try {
			document = builder.parse(this.descriptorFile).getDocumentElement();
		} catch (SAXParseException e) {
			throw new JsBuilderException("Cannot parse xml config file '"+this.descriptorFile+"': " + e.getMessage());
		} catch (SAXException e) {
			throw new JsBuilderException("Cannot parse xml config file '"+this.descriptorFile+"'");
		} catch (IOException e) {
			throw new JsBuilderException("Cannot access file '"+this.descriptorFile+"'");
		}
		
		Node moduleNode = document.getFirstChild();
		
		while (moduleNode != null){
			//LOG.debug("processing modules... " + moduleSibling.getNodeName());
			
			if (moduleNode.getNodeName().equals("module")){
				JsModule module = processModule(moduleNode);
				modules.add(module);
			}
			
			moduleNode = moduleNode.getNextSibling();
		}
		
		this.modules = modules;
	}
	
	
	
}
