/*
This file is part of the Project JsBuild

JsBuild - http://projects.stueckseln.de/jsbuild/
JsBuild is a project released under the MIT license.


Copyright (c) 2009 Thomas Lehmann

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package de.stueckseln.buildtools.jsbuild.ant;

import java.io.File;


import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

import de.stueckseln.buildtools.utils.FileHelper;
import de.stueckseln.buildtools.utils.Logger;
import de.stueckseln.buildtools.utils.TaskLogger;
import de.stueckseln.buildtools.jsbuild.JsBuilder;
import de.stueckseln.buildtools.jsbuild.JsBuilderException;

/** Ant-Task for JsBuild
  * 
  * @author Thomas Lehmann
  *
  */
public class JsBuildTask extends Task {	
	private String source = null;
	private String target = null;
	private String descriptor = null;
	
	private String orderListPostfix = null;
	private String buildPostfix = null;
	
	private boolean overwriteExisting = false;
	
	private boolean compressEnabled = true;
	private boolean showWarnings = true;
	private boolean showInfos = true;
	
	private boolean yuiNoMunge = true;
	private boolean yuiNoOpts = true;
	private boolean yuiPreserveSemicolons = true;	
	private int yuiLinebreak = 2048;
	
	
	private String appendStr = null;	
	private String prependStr = null;	
	
	
	public void setSource(String source){
		if (source == null) {
			throw new BuildException("Argument source has not to be empty");
		}
		File file = new File(source);
		if (!file.exists() || !file.isDirectory()){
			throw new BuildException("Source is either no directory or not existing");
		}		
		this.source = file.getAbsolutePath();
	}
	
	public void setTarget(String target){
		if (target == null) {
			throw new BuildException("Argument target has not to be empty");
		}
		File file = new File(target);
		if (!file.exists() || !file.isDirectory()){
			throw new BuildException("Target is either no directory or not existing");
		}		
		this.target = file.getAbsolutePath();
	}
	
	public void setDescriptor(String descriptor){
		if (descriptor == null) {
			throw new BuildException("Argument descriptor has not to be empty");
		}
		File file = new File(descriptor);
		if (!file.exists() || !file.isFile() || !file.canRead()){
			throw new BuildException("Descriptor is not readable");
		}		
		this.descriptor = file.getAbsolutePath();	
	}

	
	private void run(){
	
		try {
			Project prj = this.getProject();
			String prop = null;
			File file = null;
			
			if (source == null){
				prop = prj.replaceProperties(prj.getProperty("jsbuild.source"));
				if (prop == null) {
					throw new BuildException("Property jsbuild.source not specified");
				}
				file = new File(prop);
				if (!file.exists() || !file.isDirectory()){
					throw new BuildException("Source is either no directory or not existing");
				}
			} else {
				log("Argument source takes precedence over property jsbuild.source", Project.MSG_INFO);
			}

			if (target == null){
				prop = prj.replaceProperties(prj.getProperty("jsbuild.target"));
				if (prop == null) {
					throw new BuildException("Property jsbuild.target not specified");
				}
				file = new File(prop);
				if (!file.exists() || !file.isDirectory()){
					throw new BuildException("Target is either no directory or not existing");
				}
				target = file.getAbsolutePath();
			} else {
				log("Argument target takes precedence over property jsbuild.target", Project.MSG_INFO);
			}
			
			if (descriptor == null) {
				prop = prj.replaceProperties(prj.getProperty("jsbuild.descriptor"));
				if (prop == null) {
					throw new BuildException("Property jsbuild.descriptor not specified");
				}
				file = new File(prop);
				if (!file.exists() || !file.isFile() || !file.canRead()){
					throw new BuildException("Cannot read descriptor file.");
				}
				descriptor = file.getAbsolutePath();
			} else {
				log("Argument descriptor takes precedence over property jsbuild.descriptor", Project.MSG_INFO);
			}
				
			prop = prj.getProperty("jsbuild.build.orderlist.postfix.override");
			if (prop != null && prop.length() > 1){
				orderListPostfix = prop;
			}
			
			prop = prj.getProperty("jsbuild.build.module.postfix.override");
			if (prop != null && prop.length() > 1){
				buildPostfix = prop;
			}
			
			
			prop = prj.getProperty("jssuild.overwriteexisting");
			if (prop != null){
				overwriteExisting = Boolean.parseBoolean(prop);
			}
			
			

			
			
			prop = prj.getProperty("jsbuild.compress");
			if (prop != null){
				compressEnabled = Boolean.parseBoolean(prop);
			}
			
			prop = prj.getProperty("jsbuild.show-warnings");
			if (prop != null){
				showWarnings = Boolean.parseBoolean(prop);
			}
			
			prop = prj.getProperty("jsbuild.show-infos");
			if (prop != null){
				showInfos = Boolean.parseBoolean(prop);
			}			
	
			
			prop = prj.getProperty("jsbuild.compressor.no-munge");
			if (prop != null){
				yuiNoMunge = Boolean.parseBoolean(prop);
			}			

			prop = prj.getProperty("jsbuild.compressor.no-opts");
			if (prop != null){
				yuiNoOpts = Boolean.parseBoolean(prop);
			}			

			prop = prj.getProperty("jsbuild.compressor.preserve-semicolons");
			if (prop != null){
				yuiPreserveSemicolons = Boolean.parseBoolean(prop);
			}			

			prop = prj.getProperty("jsbuild.compressor.linebreak");
			if (prop != null){
				int val;
				try {
					val =  Integer.parseInt(prop);
				} catch (NumberFormatException e) {
					throw new BuildException("Property jsbuild.compressor.linebreak is not a valid integer: " + prop); 
				}
				
				if (val > 32768 || val < 32){
					throw new BuildException("Property jsbuild.compressor.linebreak out of range (32-32768): " + val);
				}
				
				yuiLinebreak = val;
			}			

			StringBuffer buf = null;
			
			prop = prj.replaceProperties(prj.getProperty("jsbuild.preprend.file"));
			if (prop != null){
				file = new File(prop);
				if (!file.exists() || !file.isFile() || !file.canRead()){
					throw new BuildException("Cannot read prepend file '"+file.getAbsolutePath()+"'");
				}

				try {
					this.prependStr = FileHelper.readFileToString(file);
				} catch (Exception e) {
					throw new BuildException("Cannot read prepend file '"+file.getAbsolutePath()+"'");
				}
			}
			
			
			prop = prj.replaceProperties(prj.getProperty("jsbuild.append.file"));
			if (prop != null){
				file = new File(prop);
				if (!file.exists() || !file.isFile() || !file.canRead()){
					throw new BuildException("Cannot read append file '"+file.getAbsolutePath()+"'");
				}
				buf = new StringBuffer();
				try {
					this.appendStr = FileHelper.readFileToString(file);
				} catch (Exception e) {
					throw new BuildException("Cannot read append file '"+file.getAbsolutePath()+"'");
				}
			}
			
			Logger logger = (Logger) new TaskLogger(this);
			
			JsBuilder jsBuilder = new JsBuilder(source, target, descriptor, logger);
			jsBuilder.setOverwriteExisting(overwriteExisting);
			
			if (orderListPostfix != null) {
				jsBuilder.setBuiltOrderList(orderListPostfix);
			}
			
			if (buildPostfix != null) {
				jsBuilder.setBuiltPostfix(buildPostfix);
			}
			
			jsBuilder.setCompressEnabled(compressEnabled);
			jsBuilder.setShowCompressorWarnings(showWarnings);
			jsBuilder.setShowInfos(showInfos);
			
			jsBuilder.setYuiNoMunge(yuiNoMunge);
			jsBuilder.setYuiNoOpts(yuiNoOpts);
			jsBuilder.setYuiPreserveSemicolons(yuiPreserveSemicolons);
			jsBuilder.setYuiLinebreak(yuiLinebreak);
			
			if (prependStr != null) {
				jsBuilder.setPrependString(prependStr);
			}
			
			if (appendStr != null) {
				jsBuilder.setAppendString(appendStr);
			}
			
			jsBuilder.build();			
			
			/*
			 * Debug code
			 *
			System.out.println(source);
			System.out.println(target);
			System.out.println(descriptor);

			System.out.println(orderListPostfix);
			System.out.println(buildPostfix);

			System.out.println(overwriteExisting);

			System.out.println(compressEnabled);
			System.out.println(showWarnings);
			System.out.println(showInfos);

			System.out.println(yuiNoMunge);
			System.out.println(yuiNoOpts);
			System.out.println(yuiLinebreak);
			System.out.println(yuiPreserveSemicolons);

			System.out.println("'" + prependStr + "'");
			System.out.println("'" + appendStr + "'");
			*/

			
		} catch (JsBuilderException be){
				throw new BuildException("Error on Building JS: " + be.getMessage());
		} finally {
		}
	}
	

	public void execute() throws BuildException {
		run();		
	}
	

}
