/*
This file is part of the Project JsBuild

JsBuild - http://projects.stueckseln.de/jsbuild/
JsBuild is a project released under the MIT license.


Copyright (c) 2009 Thomas Lehmann

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package de.stueckseln.buildtools.utils;

import java.util.Iterator;
import java.util.LinkedList;

/** Implemented path builder (@see {@link PathBuilder}) internally using an ArrayList.
  * This class splits a path into an array
  * 
  * @author Thomas Lehmann
  *
  */
public class SimplePathBuilder implements PathBuilder {
	private char sep;
	private LinkedList<String> pathElements; 
	
	public SimplePathBuilder(){
		this.sep = PathBuilder.DEFAULT_SEPARATOR;
		this.pathElements = new LinkedList<String>();
	}

	public SimplePathBuilder(String path) {
		this();
		this.init(path);
	}
	
	private static boolean isCurrent(String s){
		return (s.equals("."));
	}

	private static boolean isParent(String s){
		return (s.equals(".."));
	}
	
	public PathBuilder setSeparator(char newSeparator) {
		this.sep = newSeparator;
		return this;
	}

	public char getSeparator() {
		return this.sep;
	}

	/** Add another PathBuilder to this PathBuild
 	  * This implicitly will lead to a init() because the added PathBuilder denotes an absolute path   
	  * @param p another PathBuilder to add
	  * @return this PathBuilder instance 
	  */
	public PathBuilder append(PathBuilder p) {
		this.append(p.toString());
		return this;
	}
	
	/** Add a path-component to this abstract path
 	  * @param s string denoting a path-component
	  * @return this PathBuilder instance
	  */
	public PathBuilder append(String s) {
		LinkedList<String> elements = getPathElementsForString(s);
		if (elements != null && elements.size() > 0) {
			if (elements.getFirst().equals("")){
				this.init(); // first element was empty /asfd/sad -> [,asfd,sad]
							 // so init the path
				elements.removeFirst();
			}
			
			add(elements);
			
		}
		return this;
	}

	public PathBuilder append(char c) {
		append(String.valueOf(c));
		return this;
	}

	public PathBuilder append(byte b) {
		append(String.valueOf(b));
		return this;
	}

	public PathBuilder append(int i) {
		append(String.valueOf(i));
		return this;
	}

	public PathBuilder append(long l) {
		append(String.valueOf(l));
		return this;
	}

	public String getPath() {
		Iterator<String> it = this.pathElements.iterator();
		StringBuffer str = new StringBuffer();
				
		if (this.pathElements.size() == 0) {
			str.append(this.sep);
		}

		while(it.hasNext()){
			str.append(this.sep);
			str.append(it.next());
		}
		
		return str.toString();
	}

	public String getPath(boolean init) {
		String path = getPath();
		if (init) {
			init();
		}
		return path;
	}
	
	public String toString(){
		return this.getPath();
	}

	/** Remove last directory if any
	  * @return this PathBuilder
	  */
	private PathBuilder drop(){
		if (this.pathElements.size() > 0) {
			this.pathElements.removeLast();
		}
		
		return this;
	}
	
	/** Add a pre-processed list of path-elements, removing any path components of relative path's (.. and .)
	  * @param elements as LinkedList
	  * @return this PathBuilder instance  
	  */
	private PathBuilder add(LinkedList<String> elements) {
		Iterator<String> elementIt;
		String element;
		if (elements != null) {
			elementIt = elements.iterator();
					
			while (elementIt.hasNext()){
				element = elementIt.next();
				
				if (isCurrent(element)) {
					continue;
				} else if (isParent(element)){
					drop();
					continue;
				} else {
					this.pathElements.add(element);
				}				
			} // of while
		}		
		return this;
	}
	
	/** Initialize this PathBuilder by clearing internal path-element list
	  * @return this PathBuilder instance
	  */
	public PathBuilder init() {
		pathElements.clear();
		return this;
	}

	/** Initialize this PathBuilder by clearing internal path-element list and set new path
	  * @param path new path to set
	  * @return this PathBuilder instance
	  */
	public PathBuilder init(String path) {
		init();
		this.append(path);
		return this;
	}

	/** Returns the given path as list /abc/def/ghi -> [,abc,def,ghi]
	  * and ensures that the path does not contain multiple <sep>
	  * 
	  * If the path is absolute, the first element will be empty.
	  * 
	  * @param path to split
	  * @return LinkedList of path-elements
	  */
	private LinkedList<String> getPathElementsForString(String path){
		if (path == null){
			throw new NullPointerException("Path must not be null.");
		}
		
		String[] els = path.split(String.valueOf(this.sep));
		
		if (els.length == 0) {
			return null;
		}
		
		LinkedList<String> elements = new LinkedList<String>();

		for (int i=0; i<els.length; i++){
			if (i > 0 && els[i].equals("")){
				// ignore further empty elements (//abc//def/ != [,,abc,,def] !)
				continue;
			}
			elements.add(els[i]);
		}
		
		return elements;
	}

}
